@@ -23,7 +23,7 @@ impl<T: Clone> Matrix<T> {
         let item = self.matrix[m][n].clone();
 
         if item.is_some() {
-            self.clear(pos);
+            self.delete(pos).unwrap();
         }
 
         return item;
@@ -41,15 +41,31 @@ impl<T: Clone> Matrix<T> {
         return self.matrix[m][n].as_mut();
     }
 
-    pub fn set(&mut self, item: T, pos: (usize, usize)) {
+    pub fn set(&mut self, item: T, pos: (usize, usize)) -> Result<(), ()> {
+        if !self.is_in_bounds((pos.0 as isize, pos.1 as isize)) {
+            return Err(());
+        }
+
         let (m, n) = pos;
         self.matrix[m][n] = Some(item);
+        return Ok(());
     }
 
-    pub fn clear(&mut self, pos: (usize, usize)) {
+    pub fn delete(&mut self, pos: (usize, usize)) -> Result<(), ()> {
+        self.is_in_bounds((pos.0 as isize, pos.1 as isize));
+
         let (m, n) = pos;
 
         self.matrix[m][n] = None;
+
+        return Ok(());
+    }
+
+    pub fn clear(&mut self){
+        let size = (self.matrix.len(), self.matrix[0].len());
+        let new_matrix = Matrix::<T>::new(size);
+
+        self.matrix = new_matrix.matrix;
     }
 
     pub fn translate(&mut self, from: (usize, usize), dir: (isize, isize)) -> Result<(), ()> {
@@ -59,16 +75,36 @@ impl<T: Clone> Matrix<T> {
             return Err(());
         }
 
-        let to = (
-            (from.0 as isize + dir.0) as usize,
-            (from.1 as isize + dir.1) as usize,
-        );
+        let to = (from.0 as isize + dir.0, from.1 as isize + dir.1);
 
-        self.set(item.unwrap(), to);
-        self.clear(from);
+        if !self.is_in_bounds(to) {
+            return Err(());
+        }
+
+        let to = (to.0 as usize, to.1 as usize);
+
+        if self.set(item.unwrap(), to).is_err() {
+            return Err(());
+        }
+
+        if self.delete(from).is_err() {
+            return Err(());
+        }
 
         return Ok(());
     }
+
+    fn is_in_bounds(&self, pos: (isize, isize)) -> bool {
+        if pos.0 < 0 && pos.0 > (self.matrix.len() - 1) as isize {
+            return false;
+        }
+
+        if pos.1 < 0 && pos.1 > (self.matrix[0].len() - 1) as isize {
+            return false;
+        }
+
+        return true;
+    }
 }
 
 impl<T: Clone + Copy> From<Vec<Vec<T>>> for Matrix<T> {
@@ -79,7 +115,7 @@ impl<T: Clone + Copy> From<Vec<Vec<T>>> for Matrix<T> {
 
         for i in 0..m {
             for j in 0..n {
-                matrix.set(items[i][j], (i, j));
+                matrix.set(items[i][j], (i, j)).unwrap();
             }
         }
 
